#!/usr/bin/env python

import sys
import os

import tornado.template


def main():
    root = os.getcwd()
    root = os.path.abspath(root)

    template_directory = os.path.join(root, 'templates')

    loader = tornado.template.Loader(template_directory)

    trim = len(template_directory) + 1

    for (base,directories,filenames) in os.walk(template_directory):
        for filename in filenames:
            if not filename.endswith('.html'):
                continue
            path = os.path.join(base, filename)[trim:]
            content = loader.load(path).generate()

            dest = os.path.join(root, 'site', path)

            try:
                os.makedirs(os.path.dirname(dest))
            except OSError:
                pass

            open(dest, 'wb').write(content)

            print path, '=>', dest


if '__main__' == __name__:
    main()




import logging

import pyaas


def initialize(engine=None, section=None):
    if not engine:
        engine = pyaas.args.db or pyaas.config.get('storage', 'engine')

    path = 'pyaas.storage.engines.' + engine

    try:
        mod = __import__(path)
    except ImportError:
        raise pyaas.error('Unknown storage engine: %s', engine)

    for sub in path.split('.')[1:]:
        mod = getattr(mod, sub)

    try:
        dbcls = getattr(mod, 'Database')
    except AttributeError:
        raise pyaas.error('Bad storage engine: %s', engine)

    logging.info('Storage engine: %s', engine)

    if not section:
        section = engine

    options = {}
    if pyaas.config.has_section(section):
        options = dict(pyaas.config.items(section))

    logging.debug('Options: %s', options)

    pyaas.db = dbcls(**options)

    if pyaas.args.initdb:
        pyaas.db.Initialize()

    return pyaas.db




import sys
import os
import argparse
import collections
import logging

try:
    import configparser
except ImportError:
    import ConfigParser as configparser

import pyaas

pyaas.argparser = argparse.ArgumentParser()

pyaas.argparser.add_argument('--address', '-a',
    help = 'Interface to bind to')

pyaas.argparser.add_argument('--port', '-p',
    type=int,
    help='Port to bind to')

pyaas.argparser.add_argument('--newcookie',
    action='store_true',
    help='Generate a new cookie')

pyaas.argparser.add_argument('--ini', '-i',
    help='Specify additional ini file')

pyaas.argparser.add_argument('--debug', '-d',
    action='store_true',
    help='Print verbose debugging information')

logging.basicConfig(
    format  = '%(asctime)s %(levelname)-8s %(message)s',
    datefmt = '%Y-%m-%d %H:%M:%S',
    level   = 0
)


def load(program=None):
    if program is None:
        program = os.path.basename(sys.argv[0])

        if program.endswith('.py'):
            program = program.rsplit('.', 1)[0]

    pyaas.args = pyaas.argparser.parse_args()

    pyaas.config = configparser.SafeConfigParser(dict_type=collections.OrderedDict)
    pyaas.config.optionxform = str

    ini_files = [
        os.path.join(pyaas.prefix, 'etc', pyaas.namespace, program + '.ini'),
        os.path.join(pyaas.prefix, 'etc', pyaas.namespace, program + '.ini.local')
    ]

    if pyaas.args.ini:
        ini_files.append(pyaas.args.ini)

    try:
        ok = pyaas.config.read(ini_files)
    except configparser.ParsingError as e:
        raise pyaas.error('Unable to parse file: %s', e)

    if not ok:
        raise pyaas.error('Unable to read config file(s): %s', ini_files)

    # TODO: log to file

    #root = logging.getLogger()



import sys
import logging

import pyaas

try:
    import pymongo
    import gridfs
    import bson
    import bson.json_util
except ImportError:
    raise pyaas.error('Missing pymongo module')


class Database:
    def __init__(self, **kwds):
        self.dbserver = pyaas.args.dbhost  or pyaas.config.get('mongo', 'server')
        self.dbname   = pyaas.args.dbname  or pyaas.config.get('mongo', 'database')
        self.store    = pyaas.args.dbstore or pyaas.config.get('mongo', 'store')

        try:
            pyaas.mongo = pymongo.Connection(self.dbserver)
        except pymongo.errors.AutoReconnect:
            raise pyaas.error('Could not connect to database')

        if pyaas.args.reset and pyaas.args.confirm:
            self.Reset()
            sys.exit(0)
        elif pyaas.args.reset:
            logging.error('Reset issued without confirmation')
            sys.exit(-1)

        pyaas.mongo.document_class = bson.SON

        pyaas.db = pyaas.mongo[self.dbname]
        pyaas.fs = gridfs.GridFS(pyaas.mongo[self.store])

    def Initialize(self):
        return

    def Reset(self):
        pyaas.mongo.drop_database(self.dbname)
        pyaas.mongo.drop_database(self.store)

    def Find(self, table, params=None, sort=None):
        return list(pyaas.db[table].find(params, sort=sort))

    def FindOne(self, table, _id):
        return pyaas.mongo[table].find_one(_id)

    def Count(self, table):
        return 0

    def Insert(self, table, values):
        return

    def Update(self, table, values=None):
        if values:
            self.record.update(values)
        pyaas.db[table].save(self.record)
        return self

    def Remove(self, table, _id):
        pyaas.db[table].remove(self.id)
        return True




import os

import pyaas

try:
    import sqlite3
except ImportError:
    raise pyaas.error('Missing sqlite3 module')


class Database:
    def __init__(self, path=None):
        if path is None:
            path = ':memory:'
        else:
            path = os.path.join(pyaas.prefix, path)
            # TODO: make any missing directories

        try:
            self.conn = sqlite3.connect(path)
        except sqlite3.OperationalError:
            raise pyaas.error('Unable to open database: %s', path)

        self.conn.row_factory = sqlite3.Row
        self.cursor = self.conn.cursor()

    def Initialize(self):
        schema = os.path.join(pyaas.prefix, pyaas.config.get('storage', 'schema'))
        schema = open(schema, 'rb').read()
        self.cursor.executescript(schema)
        self.conn.commit()

    def Find(self, table, params=None, sort=None):
        statement = 'SELECT * FROM ' + table
        if params:
            statement += ' WHERE ' + params
        if sort:
            statement += ''

        self.cursor.execute(statement)
        return self.cursor.fetchall()

    def FindOne(self, table, _id):
        statement = 'SELECT * FROM {0} WHERE id = ?'.format(table)
        self.cursor.execute(statement, [_id])
        return self.cursor.fetchone()

    def Count(self, table):
        statement = 'SELECT count(*) FROM {0}'.format(table)
        self.cursor.execute(statement)
        return self.cursor.fetchone()[0]

    def Insert(self, table, values):
        columns = ','.join(values.keys())
        placeholder = ','.join('?' * len(values))
        statement = 'INSERT INTO {0} ({1}) VALUES ({2})'.format(table, columns, placeholder)

        try:
            self.cursor.execute(statement, values.values())
        except sqlite3.ProgrammingError:
            raise pyaas.error('Problem executing statement')

        if 'id' not in values:
            values['id'] = self.cursor.lastrowid

        self.conn.commit()

    def Update(self, table, values):
        _id = values['id']
        columns = ','.join(s + '=?' for s in values.keys())
        statement = 'UPDATE {0} SET {1} WHERE id=?'.format(table, columns, _id)
        try:
            self.cursor.execute(statement, values.values() + [_id])
        except sqlite3.ProgrammingError:
            raise pyaas.error('Problem executing statement')

        self.conn.commit()

    def Remove(self, table, _id):
        statement = 'DELETE FROM {0} WHERE id = ?'.format(table)
        self.cursor.execute(statement, [_id])
        self.conn.commit()




import tornado.web
import tornado.websocket


class Broadcast(tornado.websocket.WebSocketHandler):
    def initialize(self, websockets=None):
        if websockets is None:
            self.websockets = self.application.websockets
        else:
            self.websockets = websockets

    def open(self):
        self.stream.set_nodelay(True)

        self.websockets.add(self)

    def on_message(self, msg):
        if msg == 'ping':
            return

        for ws in self.websockets:
            if ws is self:
                continue
            ws.write_message(msg)

    def on_close(self):
        try:
            self.websockets.remove(self)
        except KeyError:
            pass


class Protected(Broadcast):
    def open(self):
        if not self.get_secure_cookie('uid'):
            raise tornado.web.HTTPError(403)

        Broadcast.open(self)



#!/usr/bin/env python

import sys
import os
import argparse
import zipfile


def main():
    argparser = argparse.ArgumentParser()

    argparser.add_argument('--name', '-n',
        default='example', required=True,
        help='Name of the project to create')

    args = argparser.parse_args()

    dstdir = os.path.join(os.getcwd(), os.path.basename(args.name))
    dstdir = os.path.abspath(dstdir)

    try:
        os.mkdir(dstdir)
    except OSError as e:
        if 17 == e.errno:
            sys.stderr.write('Directory with that name already exists.\n')
            sys.exit(-1)
        raise

    skel = os.path.dirname(__file__)
    skel = os.path.join(skel, 'skel.zip')
    skel = zipfile.ZipFile(skel, 'r')

    skel.extractall(dstdir)

    os.chdir(dstdir)

    for src in ['bin/example.py', os.path.join('etc', 'example.ini')]:
        dst = src.replace('example', args.name)
        os.rename(src, dst)


if '__main__' == __name__:
    main()
